"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc = __importStar(require("grpc"));
class CallArgument {
    constructor(server, service, method, _call) {
        this.server = server;
        this.service = service;
        this.method = method;
        this._call = _call;
        this._outHeaders = {};
        this.callStart = Date.now();
    }
    get cancelled() {
        return this._call.cancelled;
    }
    get client() {
        return this._call.getPeer();
    }
    get headers() {
        const headers = Object.assign({}, this._call.metadata.getMap());
        Object.keys(headers).forEach(name => {
            const values = this._call.metadata.get(name);
            if (Array.isArray(values) && values.length > 1) {
                headers[name] = values;
            }
        });
        return headers;
    }
    set(key, value) {
        if (value === undefined) {
            delete this._outHeaders[key];
        }
        else {
            this._outHeaders[key] = value;
        }
    }
    responseMetadata() {
        const metadata = new grpc.Metadata();
        for (const key of Object.keys(this._outHeaders)) {
            const val = this._outHeaders[key];
            if (Array.isArray(val)) {
                const [first, ...items] = val;
                metadata.set(key, first);
                for (const item of items) {
                    metadata.set(key, item);
                }
            }
            else {
                metadata.set(key, val);
            }
        }
        return metadata;
    }
}
exports.CallArgument = CallArgument;
