"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc_error_messages_1 = require("grpc-error-messages");
const callArgument_1 = require("./callArgument");
const logger_1 = __importDefault(require("./logger"));
function onError(call, request, arg, error, callback) {
    try {
        logger_1.default.error(`[error]\t${arg.client}\t${arg.service}.${arg.method}(${JSON.stringify(request)}): ${Date.now() -
            arg.callStart}ms`, error);
        if (typeof error.code !== 'number') {
            error.code = grpc_error_messages_1.ErrorCodes.INTERNAL;
        }
    }
    finally {
        callback(error);
    }
}
function callStub(server, serviceName, svcImpl, method) {
    return (call, callback) => {
        const arg = new callArgument_1.CallArgument(server, serviceName, method, call);
        const { request } = call;
        try {
            logger_1.default.debug(`[call]\t${call.getPeer()}\t${serviceName}.${method}(${JSON.stringify(request)})`);
            if (typeof svcImpl[method] !== 'function') {
                throw new grpc_error_messages_1.UnimplementedError(`The method "${method}" is not implemented.`);
            }
            Promise.resolve(svcImpl[method](request, arg))
                .then(result => {
                if (typeof result !== 'object' || result === null || result === undefined) {
                    throw new grpc_error_messages_1.InternalError(`Operation must return a valid message.`);
                }
                logger_1.default.info(`[done]\t${call.getPeer()}\t${serviceName}.${method}(${JSON.stringify(request)}): ` +
                    `${Date.now() - arg.callStart}ms`);
                call.sendMetadata(arg.responseMetadata());
                callback(null, result);
            })
                .catch(ex => {
                onError(call, request, arg, ex, callback);
            });
        }
        catch (ex) {
            onError(call, request, arg, ex, callback);
        }
    };
}
exports.callStub = callStub;
